@@ -100,31 +100,36 @@ local DEFAULTS = setmetatable({
     url = "", -- DSN url
 
     master_name = "mymaster",
-    role = "master",  -- master | slave | any
+    role = "master",  -- master | slave
     sentinels = {},
 
+    -- Redis proxies typically don't support full Redis capabilities
+    connection_is_proxied = false,
+
+    disabled_commands = {},
+
 }, fixed_field_metatable)
 
 
+-- This is the set of commands unsupported by Twemproxy
+local default_disabled_commands = {
+    "migrate", "move", "object", "randomkey", "rename", "renamenx", "scan",
+    "bitop", "msetnx", "blpop", "brpop", "brpoplpush", "psubscribe", "publish",
+    "punsubscribe", "subscribe", "unsubscribe", "discard", "exec", "multi",
+    "unwatch", "watch", "script", "auth", "echo", "select", "bgrewriteaof",
+    "bgsave", "client", "config", "dbsize", "debug", "flushall", "flushdb",
+    "info", "lastsave", "monitor", "save", "shutdown", "slaveof", "slowlog",
+    "sync", "time"
+}
+
+
 local _M = {
-    _VERSION = '0.04',
+    _VERSION = '0.06',
 }
 
 local mt = { __index = _M }
 
 
-function _M.new(config)
-    local ok, config = pcall(tbl_copy_merge_defaults, config, DEFAULTS)
-    if not ok then
-        return nil, config  -- err
-    else
-        return setmetatable({
-            config = setmetatable(config, fixed_field_metatable)
-        }, mt)
-    end
-end
-
-
 local function parse_dsn(params)
     local url = params.url
     if url and url ~= "" then
@@ -135,6 +140,7 @@ local function parse_dsn(params)
             return nil, "could not parse DSN: " .. tostring(err)
         end
 
+        -- TODO: Support a 'protocol' for proxied Redis?
         local fields
         if m[1] == "redis" then
             fields = { "password", "host", "port", "db" }
@@ -145,29 +151,58 @@ local function parse_dsn(params)
         -- password may not be present
         if #m < 5 then tbl_remove(fields, 1) end
 
-        local roles = { m = "master", s = "slave", a = "any" }
+        local roles = { m = "master", s = "slave" }
+
+        local parsed_params = {}
 
         for i,v in ipairs(fields) do
-            params[v] = m[i + 1]
+            parsed_params[v] = m[i + 1]
             if v == "role" then
-                params[v] = roles[params[v]]
+                parsed_params[v] = roles[parsed_params[v]]
             end
         end
-    end
 
-    return true, nil
+        return tbl_copy_merge_defaults(params, parsed_params)
+    end
 end
 _M.parse_dsn = parse_dsn
 
 
-function _M.connect(self, params)
-    local params = tbl_copy_merge_defaults(params, self.config)
-
-    if params.url then
-        local ok, err = parse_dsn(params)
+function _M.new(config)
+    -- Fill out gaps in config with any dsn params
+    if config and config.url then
+        local err
+        config, err = parse_dsn(config)
         if not ok then ngx_log(ngx_ERR, err) end
     end
 
+    local ok, config = pcall(tbl_copy_merge_defaults, config, DEFAULTS)
+    if not ok then
+        return nil, config  -- err
+    else
+        -- In proxied Redis mode disable default commands
+        if config.connection_is_proxied == true and
+            not next(config.disabled_commands) then
+
+            config.disabled_commands = default_disabled_commands
+        end
+
+        return setmetatable({
+            config = setmetatable(config, fixed_field_metatable)
+        }, mt)
+    end
+end
+
+
+function _M.connect(self, params)
+    if params and params.url then
+        local err
+        params, err = parse_dsn(params)
+        if not ok then ngx_log(ngx_ERR, err) end
+    end
+
+    params = tbl_copy_merge_defaults(params, self.config)
+
     if #params.sentinels > 0 then
         return self:connect_via_sentinel(params)
     else
@@ -197,7 +232,7 @@ function _M.connect_via_sentinel(self, params)
         return nil, err, previous_errors
     end
 
-    if role == "master" or role == "any" then
+    if role == "master" then
         local master, err = get_master(sentnl, master_name)
         if master then
             master.db = db
@@ -213,31 +248,32 @@ function _M.connect_via_sentinel(self, params)
                 end
             end
         end
-    end
 
-    -- We either wanted a slave, or are failing over to a slave "any"
-    local slaves, err = get_slaves(sentnl, master_name)
-    sentnl:set_keepalive()
-
-    if not slaves then
-        return nil, err
-    end
-
-    -- Put any slaves on 127.0.0.1 at the front
-    tbl_sort(slaves, sort_by_localhost)
-
-    if db or password then
-        for i,slave in ipairs(slaves) do
-            slave.db = db
-            slave.password = password
-        end
-    end
-
-    local slave, err, previous_errors = self:try_hosts(slaves)
-    if not slave then
-        return nil, err, previous_errors
     else
-        return slave
+        -- We want a slave
+        local slaves, err = get_slaves(sentnl, master_name)
+        sentnl:set_keepalive()
+
+        if not slaves then
+            return nil, err
+        end
+
+        -- Put any slaves on 127.0.0.1 at the front
+        tbl_sort(slaves, sort_by_localhost)
+
+        if db or password then
+            for i,slave in ipairs(slaves) do
+                slave.db = db
+                slave.password = password
+            end
+        end
+
+        local slave, err, previous_errors = self:try_hosts(slaves)
+        if not slave then
+            return nil, err, previous_errors
+        else
+            return slave
+        end
     end
 end
 
@@ -265,6 +301,15 @@ function _M.connect_to_host(self, host)
     local config = self.config
     r:set_timeout(config.connect_timeout)
 
+    -- Stub out methods for disabled commands
+    if next(config.disabled_commands) then
+        for _, cmd in ipairs(config.disabled_commands) do
+            r[cmd] = function(...)
+                return nil, ("Command "..cmd.." is disabled")
+            end
+        end
+    end
+
     local ok, err
     local path = host.path
     local opts = config.connection_options
@@ -296,7 +341,8 @@ function _M.connect_to_host(self, host)
             end
         end
 
-        if host.db ~= nil then
+        -- No support for DBs in proxied Redis.
+        if config.connection_is_proxied ~= true and host.db ~= nil then
             r:select(host.db)
         end
         return r, nil
@@ -307,7 +353,10 @@ end
 local function set_keepalive(self, redis)
     -- Restore connection to "NORMAL" before putting into keepalive pool,
     -- ignoring any errors.
-    redis:discard()
+    -- Proxied Redis does not support transactions.
+    if self.config.connection_is_proxied ~= true then
+        redis:discard()
+    end
 
     local config = self.config
     return redis:set_keepalive(
